@@ -9,9 +9,7 @@ import sre_constants
 import random
 from backrefs import _bre_parse
 import copy
-import warnings
 
-PY36_PLUS = (3, 6) <= sys.version_info
 PY37_PLUS = (3, 7) <= sys.version_info
 PY39_PLUS = (3, 9) <= sys.version_info
 
@@ -47,10 +45,14 @@ class TestSearchTemplate(unittest.TestCase):
         self.assertTrue(bre.match(r'[\p{digit}-Z]', '-') is not None)
         self.assertTrue(bre.match(r'[\p{digit}-Z]', '5') is not None)
 
-        self.assertTrue(bre.match(r'[0-\c]', '-') is not None)
-        self.assertTrue(bre.match(r'[0-\c]', '0') is not None)
-        self.assertTrue(bre.match(r'[0-\c]', 'A') is not None)
-        self.assertTrue(bre.match(r'[0-\c]', 'D') is not None)
+        self.assertTrue(bre.match(r'[0-\p{upper}]', '-') is not None)
+        self.assertTrue(bre.match(r'[0-\p{upper}]', '0') is not None)
+        self.assertTrue(bre.match(r'[0-\p{upper}]', 'A') is not None)
+        self.assertTrue(bre.match(r'[0-\p{upper}]', 'D') is not None)
+
+        self.assertTrue(bre.match(r'[\h-Z]', 'Z') is not None)
+        self.assertTrue(bre.match(r'[\h-Z]', '\x20') is not None)
+        self.assertTrue(bre.match(r'[\h-Z]', '-') is not None)
 
     def test_non_raw_string_unicode(self):
         """Test non raw string Unicode notation."""
@@ -100,13 +102,12 @@ class TestSearchTemplate(unittest.TestCase):
     def test_not_flags(self):
         """Test invalid flags."""
 
-        if PY36_PLUS:
-            with pytest.raises(sre_constants.error):
-                bre.compile(r'(?-q:test)')
+        with pytest.raises(sre_constants.error):
+            bre.compile(r'(?-q:test)')
 
-            if not PY37_PLUS:
-                with pytest.raises(sre_constants.error):
-                    bre.compile(r'(?a:test)')
+        if not PY37_PLUS:
+            with pytest.raises(sre_constants.error):
+                bre.compile(r'(?a:test)')
 
     def test_comment_failures(self):
         """Test comment failures."""
@@ -179,15 +180,8 @@ class TestSearchTemplate(unittest.TestCase):
             r"test\b(?<=\w)"
         )
 
-        if PY36_PLUS:
-            with pytest.raises(sre_constants.error):
-                bre.compile_search(r'[\m]test')
-        else:
-            pattern = bre.compile_search(r'[\m]test')
-            self.assertEqual(
-                pattern.pattern,
-                r"[\m]test"
-            )
+        with pytest.raises(sre_constants.error):
+            bre.compile_search(r'[\m]test')
 
     def test_cache(self):
         """Test cache."""
@@ -213,9 +207,9 @@ class TestSearchTemplate(unittest.TestCase):
 
         with pytest.raises(_bre_parse.LoopException):
             bre.compile_search(r'((?a)(?u))')
-        if PY36_PLUS:
-            with pytest.raises(_bre_parse.LoopException):
-                bre.compile_search(r'(?-x:(?x))', re.VERBOSE)
+
+        with pytest.raises(_bre_parse.LoopException):
+            bre.compile_search(r'(?-x:(?x))', re.VERBOSE)
 
     def test_unicode_ascii_swap(self):
         """Test Unicode ASCII swapping."""
@@ -229,25 +223,24 @@ class TestSearchTemplate(unittest.TestCase):
     def test_comments_with_scoped_verbose(self):
         """Test scoped verbose with comments (Python 3.6+)."""
 
-        if PY36_PLUS:
-            pattern = bre.compile_search(
-                r'''(?u)Test # \e(?#\e)(?x:
-                Test #\e(?#\e)
-                (Test # \e
-                )Test #\e
-                )Test # \e'''
-            )
+        pattern = bre.compile_search(
+            r'''(?u)Test # \e(?#\e)(?x:
+            Test #\e(?#\e)
+            (Test # \e
+            )Test #\e
+            )Test # \e'''
+        )
 
-            self.assertEqual(
-                pattern.pattern,
-                r'''(?u)Test # \x1b(?#\e)(?x:
-                Test #\\e(?#\\e)
-                (Test # \\e
-                )Test #\\e
-                )Test # \x1b'''
-            )
+        self.assertEqual(
+            pattern.pattern,
+            r'''(?u)Test # \x1b(?#\e)(?x:
+            Test #\\e(?#\\e)
+            (Test # \\e
+            )Test #\\e
+            )Test # \x1b'''
+        )
 
-            self.assertTrue(pattern.match('Test # \x1bTestTestTestTest # \x1b') is not None)
+        self.assertTrue(pattern.match('Test # \x1bTestTestTestTest # \x1b') is not None)
 
     def test_byte_string_named_chars(self):
         """Test byte string named char."""
@@ -308,7 +301,7 @@ class TestSearchTemplate(unittest.TestCase):
             r'''(?xu)
             Test # \p{XDigit}
             (Test (?#\p{XDigit}))
-            Test \p{XDigit}
+            Test \p{PosixXDigit}
             '''
         )
 
@@ -465,90 +458,6 @@ class TestSearchTemplate(unittest.TestCase):
         pattern = bre.compile_search(r'Test [:graph:]]')
         self.assertEqual(pattern.pattern, r'Test [:graph:]]')
 
-    def test_ascii_upper_props(self):
-        """Test ASCII uppercase properties."""
-
-        pattern = bre.compile_search(br'EX\c+LE')
-        m = pattern.match(br'EXAMPLE')
-        self.assertTrue(m is not None)
-
-    def test_ascii_upper_props_group(self):
-        """Test ASCII uppercase properties in a character group."""
-
-        pattern = bre.compile_search(br'EX[\c]+LE')
-        m = pattern.match(br'EXAMPLE')
-        self.assertTrue(m is not None)
-
-    def test_ascii_lower_props(self):
-        """Test ASCII lowercase properties."""
-
-        pattern = bre.compile_search(br'EX\l+LE')
-        m = pattern.match(br'EXampLE')
-        self.assertTrue(m is not None)
-
-    def test_ascii_lower_props_group(self):
-        """Test ASCII uppercase properties in a char group."""
-
-        pattern = bre.compile_search(br'EX[\l]+LE')
-        m = pattern.match(br'EXampLE')
-        self.assertTrue(m is not None)
-
-    def test_ascii_props_mixed_group(self):
-        """Test mixed ASCII properties in group."""
-
-        pattern = bre.compile_search(br'EX[\l\c]+LE')
-        m = pattern.match(br'EXaMpLE')
-        self.assertTrue(m is not None)
-
-    def test_ascii_props_mixed(self):
-        """Test mixed ASCII properties."""
-
-        pattern = bre.compile_search(br'EX\l\c\lLE')
-        m = pattern.match(br'EXaMpLE')
-        self.assertTrue(m is not None)
-
-    def test_reverse_ascii_lower_props(self):
-        """Test reverse ASCII lowercase properties."""
-
-        pattern = bre.compile_search(br'EX\L+LE')
-        m = pattern.match(br'EXAMPLE')
-        self.assertTrue(m is not None)
-
-    def test_reverse_ascii_lower_props_group(self):
-        """Test reverse ASCII lowercase properties in a group."""
-
-        pattern = bre.compile_search(br'EX[\L]+LE')
-        m = pattern.match(br'EXAMPLE')
-        self.assertTrue(m is not None)
-
-    def test_reverse_ascii_upper_props(self):
-        """Test reverse ASCII uppercase properties."""
-
-        pattern = bre.compile_search(br'EX\C+LE')
-        m = pattern.match(br'EXampLE')
-        self.assertTrue(m is not None)
-
-    def test_reverse_ascii_upper_props_group(self):
-        """Test reverse ASCII uppercase properties in a group."""
-
-        pattern = bre.compile_search(br'EX[\C]+LE')
-        m = pattern.match(br'EXampLE')
-        self.assertTrue(m is not None)
-
-    def test_reverse_ascii_props_mixed_group(self):
-        """Test reverse mixed ASCII properties in a group."""
-
-        pattern = bre.compile_search(br'EX[\C\L]+LE')
-        m = pattern.match(br'EXaMpLE')
-        self.assertTrue(m is not None)
-
-    def test_reverse_ascii_props_mixed(self):
-        """Test reverse ASCII properties."""
-
-        pattern = bre.compile_search(br'EX\C\L\CLE')
-        m = pattern.match(br'EXaMpLE')
-        self.assertTrue(m is not None)
-
     def test_unrecognized_backrefs(self):
         """Test unrecognized backrefs."""
 
@@ -635,57 +544,6 @@ class TestSearchTemplate(unittest.TestCase):
         result = _bre_parse._SearchParser(r'\y \\y \\\y \\\\y \\\\\y').parse()
         self.assertEqual(r'\y \\y \\\y \\\\y \\\\\y', result)
 
-    def test_unicode_shorthand_properties_capital(self):
-        """
-        Exercising that Unicode properties are built correctly by testing shorthand lower and upper.
-
-        We want to test uppercase and make sure things make sense,
-        and then test lower case later.  Not extensive, just making sure its generally working.
-        """
-
-        pattern = bre.compile_search(r'EX\cMPLE', re.UNICODE)
-        m = pattern.match(r'EXÁMPLE')
-        self.assertTrue(m is not None)
-        m = pattern.match(r'exámple')
-        self.assertTrue(m is None)
-
-    def test_unicode_shorthand_properties_lower(self):
-        """Exercise the Unicode shorthand properties for lower case."""
-
-        pattern = bre.compile_search(r'ex\lmple', re.UNICODE)
-        m = pattern.match('exámple')
-        self.assertTrue(m is not None)
-        m = pattern.match('EXÁMPLE')
-        self.assertTrue(m is None)
-
-    def test_unicode_shorthand_properties_in_char_group(self):
-        """Exercise the Unicode shorthand properties inside a char group."""
-
-        pattern = bre.compile_search(r'ex[\l\c]mple', re.UNICODE)
-        m = pattern.match('exámple')
-        self.assertTrue(m is not None)
-        m = pattern.match('exÁmple')
-        self.assertTrue(m is not None)
-
-    def test_unicode_shorthand_properties_with_string_flag(self):
-        """Exercise the Unicode shorthand properties with an re string flag `(?u)`."""
-
-        pattern = bre.compile_search(r'ex[\l\c]mple(?u)')
-        m = pattern.match('exámple')
-        self.assertTrue(m is not None)
-        m = pattern.match('exÁmple')
-        self.assertTrue(m is not None)
-
-    def test_unicode_shorthand_ascii_only(self):
-        """Ensure that when the Unicode flag is not used, only ASCII properties are used."""
-
-        flags = bre.ASCII
-        pattern = bre.compile_search(r'ex\lmple', flags)
-        m = pattern.match('exámple')
-        self.assertTrue(m is None)
-        m = pattern.match('example')
-        self.assertTrue(m is not None)
-
     def test_unicode_properties_capital(self):
         """
         Exercising that Unicode properties are built correctly.
@@ -772,6 +630,15 @@ class TestSearchTemplate(unittest.TestCase):
         m = pattern.match(r'P')
         self.assertTrue(m is not None)
 
+    def test_unicode_properties_inverse_value_inverse(self):
+        """Exercising inverse Unicode properties that are using inverse values."""
+
+        pattern = bre.compile_search(r'\P{^Po}', re.UNICODE)
+        m = pattern.match(r'·')
+        self.assertTrue(m is not None)
+        m = pattern.match(r'P')
+        self.assertTrue(m is None)
+
     def test_negated_unicode_properties_inverse(self):
         """Exercising negated inverse Unicode properties."""
 
@@ -998,7 +865,7 @@ class TestReplaceTemplate(unittest.TestCase):
     def test_format_failures(self):
         """Test format parsing failures."""
 
-        with pytest.raises(sre_constants.error if PY36_PLUS else IndexError):
+        with pytest.raises(sre_constants.error):
             bre.subf('test', r'{1.}', 'test', bre.FORMAT)
 
         with pytest.raises(IndexError):
@@ -1019,7 +886,7 @@ class TestReplaceTemplate(unittest.TestCase):
         with pytest.raises(SyntaxError):
             bre.subf('test', r'test { test', 'test', bre.FORMAT)
 
-        with pytest.raises(sre_constants.error if PY36_PLUS else IndexError):
+        with pytest.raises(sre_constants.error):
             bre.subf(b'test', br'{1.}', b'test', bre.FORMAT)
 
         with pytest.raises(IndexError):
@@ -1157,14 +1024,8 @@ class TestReplaceTemplate(unittest.TestCase):
     def test_line_break_in_group(self):
         """Test that line break in group matches a normal R."""
 
-        if PY36_PLUS:
-            with pytest.raises(sre_constants.error):
-                bre.sub(r"[\R]", 'l', 'Rine\r\nRine\nRine\r')
-        else:
-            self.assertEqual(
-                bre.sub(r"[\R]", 'l', 'Rine\r\nRine\nRine\r'),
-                'line\r\nline\nline\r'
-            )
+        with pytest.raises(sre_constants.error):
+            bre.sub(r"[\R]", 'l', 'Rine\r\nRine\nRine\r')
 
     def test_horizontal_ws(self):
         """Test horizontal whitespace."""
@@ -2137,37 +1998,29 @@ class TestExceptions(unittest.TestCase):
         with pytest.raises(ValueError) as e:
             bre.compile_search(r'[[:bad:]]', re.UNICODE)
 
-        self.assertEqual(str(e.value), 'Invalid POSIX property!')
+        self.assertEqual(str(e.value), "'bad' does not appear to be a valid property")
 
     def test_bad_bytes(self):
         """Test bad bytes."""
 
-        with pytest.raises(ValueError) as e:
+        with pytest.raises(ValueError):
             bre.compile_search(r'\p{bad_bytes:n}', re.UNICODE)
 
-        self.assertEqual(str(e.value), 'Invalid Unicode property!')
-
-        with pytest.raises(ValueError) as e:
+        with pytest.raises(ValueError):
             bre.compile_search(r'\p{bad_bytes:y}', re.UNICODE)
 
-        self.assertEqual(str(e.value), 'Invalid Unicode property!')
-
     def test_bad_category(self):
         """Test bad category."""
 
-        with pytest.raises(ValueError) as e:
+        with pytest.raises(ValueError):
             bre.compile_search(r'\p{alphanumeric: bad}', re.UNICODE)
 
-        self.assertEqual(str(e.value), 'Invalid Unicode property!')
-
     def test_bad_short_category(self):
         """Test bad category."""
 
-        with pytest.raises(ValueError) as e:
+        with pytest.raises(ValueError):
             bre.compile_search(r'\pQ', re.UNICODE)
 
-        self.assertEqual(str(e.value), 'Invalid Unicode property!')
-
     def test_switch_from_format_auto(self):
         """Test a switch from auto to manual format."""
 
@@ -2407,13 +2260,34 @@ class TestConvenienceFunctions(unittest.TestCase):
     def test_match(self):
         """Test that `match` works."""
 
-        m = bre.match(r'This is a test for m[\l]+!', "This is a test for match!")
+        m = bre.match(r'This is a test for m[[:lower:]]+!', "This is a test for match!")
         self.assertTrue(m is not None)
 
-        p = bre.compile(r'This is a test for m[\l]+!')
+        p = bre.compile(r'This is a test for m[[:lower:]]+!')
         m = p.match("This is a test for match!")
         self.assertTrue(m is not None)
 
+    def test_posix_value(self):
+        """Test posix values."""
+
+        m = bre.match(r'This is a test for m[[:lower=t:]]+!', "This is a test for match!")
+        self.assertTrue(m is not None)
+
+        m = bre.match(r'This is a test for m[[:lower:t:]]+!', "This is a test for match!")
+        self.assertTrue(m is not None)
+
+    def test_incomplete_posix_value(self):
+        """Test incomplete POSIX value."""
+
+        with pytest.raises(re.error):
+            bre.match(r'This is a test for m[[:lower=t', "This is a test for m[[:lower=t")
+
+        with pytest.raises(re.error):
+            bre.match(r'This is a test for m[[:lower=t:', "This is a test for m[[:lower=t:")
+
+        with pytest.raises(re.error):
+            bre.match(r'This is a test for m[[:lower=t: ', "This is a test for m[[:lower=t: ")
+
     def test_fullmatch(self):
         """Test that `fullmatch` works."""
 
@@ -2555,7 +2429,7 @@ class TestConvenienceFunctions(unittest.TestCase):
     def test_expand(self):
         """Test that `expand` works."""
 
-        pattern = bre.compile_search(r'(This is a test for )(m[\l]+!)')
+        pattern = bre.compile_search(r'(This is a test for )(m[[:lower:]]+!)')
         m = bre.match(pattern, "This is a test for match!")
         self.assertEqual(
             bre.expand(m, r'\1\C\2\E'),
@@ -2606,55 +2480,3 @@ class TestConvenienceFunctions(unittest.TestCase):
                 p.sub(r'\ltest', 'tests')
         else:
             self.assertEqual(p.sub(r'\ltest', 'tests'), r'\ltest')
-
-
-class TestDeprecated(unittest.TestCase):
-    """Test deprecated."""
-
-    def test_lowercase(self):
-        """Test deprecated lower."""
-
-        bre.purge()
-        with warnings.catch_warnings(record=True) as w:
-            # Cause all warnings to always be triggered.
-            warnings.simplefilter("always")
-
-            bre.compile(r'\l')
-            self.assertTrue(len(w) == 1)
-            self.assertTrue(issubclass(w[-1].category, DeprecationWarning))
-
-    def test_inverse_lowercase(self):
-        """Test inverse deprecated lower."""
-
-        bre.purge()
-        with warnings.catch_warnings(record=True) as w:
-            # Cause all warnings to always be triggered.
-            warnings.simplefilter("always")
-
-            bre.compile(r'\L')
-            self.assertTrue(len(w) == 1)
-            self.assertTrue(issubclass(w[-1].category, DeprecationWarning))
-
-    def test_uppercase(self):
-        """Test deprecated upper."""
-
-        bre.purge()
-        with warnings.catch_warnings(record=True) as w:
-            # Cause all warnings to always be triggered.
-            warnings.simplefilter("always")
-
-            bre.compile(r'\c')
-            self.assertTrue(len(w) == 1)
-            self.assertTrue(issubclass(w[-1].category, DeprecationWarning))
-
-    def test_inverse_uppercase(self):
-        """Test inverse deprecated upper."""
-
-        bre.purge()
-        with warnings.catch_warnings(record=True) as w:
-            # Cause all warnings to always be triggered.
-            warnings.simplefilter("always")
-
-            bre.compile(r'\C')
-            self.assertTrue(len(w) == 1)
-            self.assertTrue(issubclass(w[-1].category, DeprecationWarning))
